/*
 * Autonomous system - Autonom prosess for posisjonering av servicepunkt og transport av objekter
 * Part: Drone
 *
 * ELE300 MAS302 - Hovedprosjekt - UiA 2012 INGELEKTRO INGMEKA
 *
 */

#include "definitions.h"

// Signals for process communication
Signal
	start_drone,
	stop_drone,
	reset_batterycheck,
	on_symbol,
	blob_data,
	symbol_correction_done,
	correction_switch,
	end_point_dock_start,
	end_point_dock_complete,
	service_dock_start,
	service_dock_complete,
	charging_needed,
	charging_done,
	object_in_box,
	no_object_in_box,
	control_command,
	line_correction_data,
	symbol_correction_data;

// Variables
int DEBUG = 1;
int	INIT_DEBUG = 1;
int	CONTROL_DEBUG = 0;
int	CORR_DEBUG = 0;
int	ENDPOINT_DEBUG = 0;
int	IR_DEBUG = 0;
int	NAV_DEBUG = 0;
int	SENSOR_DEBUG = 0;
int	SERVICE_DEBUG = 0;
int	SYM_REC_DEBUG = 0;

// Debug message configuration
void debug_set(char debug_type, char value)
{
	switch(debug_type)
	{
		case 0:
			DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting DEBUG to : %d\r\n", value);
			break;
		case 1:
			INIT_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting INIT_DEBUG to : %d\r\n", value);
			break;
		case 2:
			CONTROL_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting CONTROL_DEBUG to : %d\r\n", value);
			break;
		case 3:
			CORR_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting CORR_DEBUG to : %d\r\n", value);
			break;
		case 4:
			ENDPOINT_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting ENDPOINT_DEBUG to : %d\r\n", value);
			break;
		case 5:
			IR_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting IR_DEBUG to : %d\r\n", value);
			break;
		case 6:
			NAV_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting NAV_DEBUG to : %d\r\n", value);
			break;
		case 7:
			SENSOR_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting SENSOR_DEBUG to : %d\r\n", value);
			break;
		case 8:
			SERVICE_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting SERVICE_DEBUG to : %d\r\n", value);
			break;
		case 9:
			SYM_REC_DEBUG = value;
			usartPrint(DEBUG_PORT, "Setting SERVICE_DEBUG to : %d\r\n", value);
			break;
		default:
			usartPrint(DEBUG_PORT, "DEBUG type not used\r\n");
			break;
	}
}

int main(void)
{
	// Variables
	static int init = 0;

	// Create process signals
	start_drone = newSignal(0);
	stop_drone = newSignal(0);
	reset_batterycheck = newSignal(0);
	on_symbol = newSignal(1, sizeof(LinesymbolType));
	blob_data =  newSignal(12, sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int), sizeof(int));
	symbol_correction_done = newSignal(0);
	correction_switch= newSignal(2,sizeof(SwitchType),sizeof(SwitchType));
	end_point_dock_start = newSignal(0);
	end_point_dock_complete = newSignal(0);
	service_dock_start = newSignal(1,sizeof(ServiceType));
	service_dock_complete = newSignal(0);
	charging_needed = newSignal(0);
	charging_done = newSignal(0);
	object_in_box = newSignal(0);
	no_object_in_box = newSignal(0);
	control_command = newSignal(2, sizeof(ControlcommandType), sizeof(int));
	line_correction_data = newSignal(2, sizeof(double), sizeof(double));
	symbol_correction_data  = newSignal(3, sizeof(double), sizeof(double), sizeof(double));

	// Initialize
	if (!init)
	{
		if(INIT_DEBUG) usartPrint(DEBUG_PORT, "Starting drone process...\r\n");
		init = 1;
	}

	// Cycle through all the processes in a endless loop
	while (1) 
	{
		/* Main program */
		/*
		communicator();
		correction();
		ircamera();
		symbol_recognizer();
		sensormonitor();
		endpoint_dock();
		service_dock();
		navigator();
		control();
		*/

		/* Testing field */

		/* IR camera test processlist */
		//ircamera_test();
		//symbol_recognizer();

		/* Control test processlist */
		//control_test();
		//control();

		/* Flight test processlist */
		//terminal_test();
		flight_test_control_commands();
		control();
		//ircamera();
		//symbol_recognizer();
		//correction();

		/* Navigator test processlist */
		//navigator_test_cmds();
		//ircamera();
		//symbol_recognizer();
		//navigator();

		/* Sensor test */
		//sensor_test_cmds();
		//sensormonitor();
	}

	return 0;
} // end main
