/*
 * Communicator process
 *
 */

#include "definitions.h"

#define SERIAL_OUTBOUND 0
#define SERIAL_INBOUND 3 // 0 usb 3 xbee
#define BAUD_RATE 57600
#define MAX_CMD 5
#define MAX_MSG 120

void communicator(void)
{
	static int init = 0;
	static int i = 0;
	static char c;
    static char string[MAX_CMD];
    static char code, c_type, value;

	if (!init)
	{
		//usartInstall(SERIAL_OUTBOUND, BAUD_RATE); // Outbound from robot (On-board Arduino)
		//usartPrint(SERIAL_OUTBOUND, "Communicator init SERIAL_OUTBOUND");
		usartInstall(SERIAL_INBOUND, BAUD_RATE); // Inbound from computer (Xbee)
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Communicator - init\r\n");
		init = 1;
	}

	// Receive commands from computer over Xbee link
	if (usartDataIsPresent(SERIAL_INBOUND))
	{
		memset(string, '\0', MAX_CMD);
		do
		{
			c = usartRead(SERIAL_INBOUND);

			string[i] = c;
			if (i < (MAX_CMD - 1))
				i++;
		} while (c != '\n');

	    code = string[0];
	    c_type = (string[1]-'0');
	    value = (string[2]-'0');

	    i = 0;

	    switch (code)
		{
	    	case 's':
	    		if (c_type)
	    		{
	    			sendSignal(stop_drone);
	    		}
	    		else
	    		{
	    			sendSignal(start_drone);
	    		}
	    		break;
	    	//debug setting
			case 'x':
				if ((c_type >= 0) && (c_type < 7 ))
				{
					debug_set(c_type, value);
				}
				else
				{
					usartPrint(DEBUG_PORT, "DEBUG: Not a viable command!\r\n");
				}
				break;
			default:
				usartPrint(SERIAL_INBOUND, "Not a viable command!\r\n");
				break;
		}
	}

} //end communicator
