/* 
 * System wide definitions of signals and variables
 *
 */

#ifndef DEFINITIONS_H
#define DEFINITIONS_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <arduino/adc.h>
#include <arduino/pins.h>
#include <arduino/signals.h>
#include <arduino/timer.h>
#include <arduino/usarts.h>
#include <arduino/pwm.h>

#include "symbol_recognizer.h"
#include "i2cmaster.h"
#include "ircamera.h"
#include "navigator.h"
#include "correction.h"
#include "communicator.h"
#include "control.h"
#include "endpoint_dock.h"
#include "service_dock.h"
#include "sensormonitor.h"

#include "testing.h"

// Datatypes definitions
typedef enum {LS_None, LS_Line, LS_Service, LS_iService, LS_Endpoint, LS_iEndpoint}LinesymbolType;
typedef enum {CC_Pitch, CC_Roll, CC_Throttle, CC_Flightmode, CC_Yaw} ControlcommandType;
typedef enum {L_Service, L_Service_dock, L_iService, L_iService_dock, L_To_supply, L_Supply, L_iSupply, L_From_supply, L_To_delivery, L_Delivery, L_iDelivery, L_From_delivery, L_No_orientation} LocationType;
typedef enum {Off, On} SwitchType;
typedef enum {S_Standby, S_Charge}ServiceType;

typedef struct {
    int x;
    int y;
    int size;
} Blob;

// Signals between processes
extern Signal
	start_drone,
	stop_drone,
	reset_batterycheck,
	on_symbol,
	blob_data,
	symbol_correction_done,
	correction_switch,
	end_point_dock_start,
	end_point_dock_complete,
	service_dock_start,
	service_dock_complete,
	charging_needed,
	charging_done,
	object_in_box,
	no_object_in_box,
	control_command,
	line_correction_data,
	symbol_correction_data;

// Message ports
#define MSG_PORT 3 // 0 usb 3 xbee
#define DEBUG_PORT 3 // 0 usb 3 xbee

// Debug mode
extern int 	DEBUG,
			INIT_DEBUG,
			CONTROL_DEBUG,
			CORR_DEBUG,
			ENDPOINT_DEBUG,
			IR_DEBUG,
			NAV_DEBUG,
			SENSOR_DEBUG,
			SERVICE_DEBUG,
			SYM_REC_DEBUG;

void debug_set(char debug_type, char value);

#endif
