/*
 * Endpoint docking process
 *
 */

#include "definitions.h"

// Height to fly to securely dock to the endpoints
#define DOCKING_HEIGHT 2

// Normal flight height
#define NORMAL_HEIGHT 1

// Waiting period to reach normal height
#define N_TIME 3000

// Increase flying height
static void fly_up(void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	// Signal with the correct information
	//control_cmd = CC_Throttle;
	control_value = DOCKING_HEIGHT;
	sendSignal(control_command, &control_cmd, &control_value);
}

// Decrease flying height
static void fly_down(void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	// Signal with the correct information
	//control_cmd = CC_Throttle;
	control_value = NORMAL_HEIGHT;
	sendSignal(control_command, &control_cmd, &control_value);
}

void endpoint_dock(void)
{
	static int init = 0;
	static enum { wait_cmd, wait_obj_ch, wait_timer } order;
	static char object_flag;
	static Timer t1;

	// Initialize
	if (!init)
	{
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Endpoint dock - init\r\n");
		object_flag = 0;
		order = wait_cmd;
		init = 1;
	}

	// Start sequence of events to dock to endpoint
	switch(order)
	{
		case wait_cmd:
			if(receivedSignal(end_point_dock_start))
			{
				if (DEBUG && ENDPOINT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Endpoint dock - Started\r\n");
				fly_up();
				order = wait_obj_ch;
				emptySignalQueue(object_in_box);
				emptySignalQueue(no_object_in_box);
			}
			break;
		case wait_obj_ch:
			if(receivedSignal(object_in_box))
			{
				if (DEBUG && ENDPOINT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Endpoint dock - Received object\r\n");
				object_flag = 1;
				fly_down();
				startTimer(N_TIME, t1);
				order = wait_timer;
			}
			else if (receivedSignal(no_object_in_box))
			{
				if (DEBUG && ENDPOINT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Endpoint dock - Delivered object\r\n");
				object_flag = 0;
				fly_down();
				startTimer(N_TIME, t1);
				order = wait_timer;
			}
			break;
		case wait_timer:
			if(timeout(t1))
			{
				if (DEBUG && ENDPOINT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Endpoint dock - Complete\r\n");
				sendSignal(end_point_dock_complete);
				order = wait_cmd;
			}
			break;
	}
}
