/*
 * Navigator process
 *
 */
#include <arduino/usarts.h>
#include "definitions.h"

// Speeds to perform the needed aerodynamic operations
// Element 5 equals stop, 4 and 6 some movement, 3 and 7 even more movement, 2 and 8 extreme movement
// See control.c for details on drone control properties
#define STOP_FLYING 5
#define START_FLYING 6
#define ROTATE 4
#define STOP_ROTATE 5

void start_flying (void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	// Signal with the correct information
	//control_cmd = CC_Pitch;
	//control_value = START_FLYING;
	//sendSignal(control_command, &control_cmd, &control_value);
	if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Start flying sent\r\n");
}

void stop_flying (void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	// Signal with the correct information
	//control_cmd = CC_Pitch
	//control_value = STOP_FLYING;
	//sendSignal(control_command, &control_cmd, &control_value);
	if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Stop flying sent\r\n");
}

void rotate(int rotating_value)
{
	static int control_value;
	static ControlcommandType control_cmd;
	static char rotating = 0;

	//control_cmd = CC_Yaw;
	//if (DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Rotating start/stop sent\r\n");
	if (rotating_value && (rotating_value != rotating))
	{
		rotating_value = rotating;
		//control_cmd = CC_Yaw;
		//control_value = ROTATE;
		//sendSignal(control_command, &control_cmd, &control_value);
		if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Rotating start sent\r\n");
	}
	else if (rotating_value != rotating)
	{
		rotating_value = rotating;
		//control_cmd = CC_Yaw;
		//control_value = STOP_ROTATE;
		//sendSignal(control_command, &control_cmd, &control_value);
		if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Rotating stop sent\r\n");
	}
}


void navigator(void)
{
	static int init = 0;
	static LocationType current_location;
	static enum {D_Supply, D_Delivery} destination;
	static LinesymbolType current_symbol;
	static SwitchType line_corr_switch, symb_corr_switch;
	static ServiceType stype;

	static char go_standby = 0;
	static char chargingneeded = 0;

	static char start_flag = 0;

	// Initialize
	if (!init)
	{
		current_location = L_No_orientation;
		destination = D_Supply;
		line_corr_switch = Off;
		symb_corr_switch = Off;
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - init\r\n");
		init = 1;
	}

	switch(current_location)
	{
		case L_No_orientation:
			// Found servicestation symbol, ready to figure out the right orientation
			if(receivedSignal(on_symbol, &current_symbol) && start_flag)
			{
				switch(current_symbol)
				{
					case LS_Service:
						current_location = L_Service;
						rotate(0);
						emptySignalQueue(on_symbol);
						// Turn Symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Had No_orientation, found LS_Service\r\n");
						break;
					case LS_iService:
						current_location = L_iService;
						rotate(0);
						emptySignalQueue(on_symbol);
						// Turn Symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Had No_orientation, found LS_iService\r\n");

						break;
					default:
						break;
				}
			}

			// Docking complete, start to rotate 180 degrees
			else if (receivedSignal(service_dock_complete))
			{
				start_flag = 1;
				rotate(1);
			}

			break;

		case L_Service_dock:
			// Position drone right on top of symbol - docking
			if(receivedSignal(symbol_correction_done))
			{
				// Turn Symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

				// Check if standby is requested
				if(go_standby)
				{
					stype = S_Standby;
				}
				// Go into charging mode
				else
				{
					stype = S_Charge;
				}
				//sendSignal(service_dock_start, &stype);
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Service going to start Charging\r\n");
			}
			// Done charging or request to go out of standby, take-off
			else if (receivedSignal(service_dock_complete))
			{
				go_standby = 0;
				chargingneeded = 0;
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Service Charging done\r\n");
				line_corr_switch = Off;
				symb_corr_switch = On;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
				current_location = L_Service;
			}
			break;

		case L_Service:
			switch(destination)
			{
				case D_Supply:
					// Correction is done and the drone is correctly orientated on top of the symbol
					if(receivedSignal(symbol_correction_done))
					{
						// Turn Symbol correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						// Fly forward
						start_flying();
						current_location = L_To_supply;
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Service going to Supply\r\n");
					}
					break;
				case D_Delivery:
					// Correction is done and the drone is correctly orientated on top of the symbol
					if(receivedSignal(symbol_correction_done))
					{
						// Turn symbol correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						// Start rotating
						rotate(1);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Service going to L_iService then going to Delivery\r\n");
					}
					else if(receivedSignal(on_symbol, &current_symbol))
					{
						switch(current_symbol)
						{
							case LS_iService:
								// Stop rotating
								rotate(0);
								emptySignalQueue(on_symbol);
								current_location = L_iService;
								// Turn Symbol correction On
								line_corr_switch = Off;
								symb_corr_switch = On;
								//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

								if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Service found L_iService\r\n");
								break;
							default:
								break;
						}
					}
					break;
			}
			break;

		case L_iService_dock:
			// Position drone right on top of symbol, docking
			if(receivedSignal(symbol_correction_done))
			{
				// Turn Symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

				// Check if standby is requested
				if(go_standby)
				{
					stype = S_Standby;
				}
				// Go into charging mode
				else
				{
					stype = S_Charge;
				}
				//sendSignal(service_dock_start, &stype);
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - L_iService_dock going to start Charging\r\n");
			}
			// Done charging or request to go out of standby, take-off
			else if (receivedSignal(service_dock_complete))
			{
				go_standby = 0;
				chargingneeded = 0;
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - L_iService_dock Charging done\r\n");
				line_corr_switch = Off;
				symb_corr_switch = On;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
				current_location = L_iService;
			}
			break;

		case L_iService:
			switch(destination)
			{
				case D_Supply:
					// Correction is done and the drone is correctly oriented on top of the symbol
					if(receivedSignal(symbol_correction_done))
					{
						// Turn symbol correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						// Start rotating 180 degrees
						rotate(1);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_iService going to L_Service then going to Supply\r\n");
					}
					else if(receivedSignal(on_symbol, &current_symbol))
					{
						switch(current_symbol)
						{
							case LS_Service:
								rotate(0);
								emptySignalQueue(on_symbol);
								current_location = L_Service;
								// turn on Symbol correction On
								line_corr_switch = Off;
								symb_corr_switch = On;
								sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
								if (NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_iService found L_Service\r\n");
								break;
							default:
								break;
						}
					}
					break;
				case D_Delivery:
					// Correction is done and the drone is correctly oriented on top of the symbol
					if(receivedSignal(symbol_correction_done))
					{
						// Turn Symbol correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						// 	Fly forward
						start_flying();
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_iService going to Delivery\r\n");
						current_location = L_To_delivery;
					}
					break;
			}
			break;

		case L_To_supply:
			if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_Line:
						// Turn Line correction On
						line_corr_switch = On;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_to_supply found LS_line to Supply\r\n");
						break;
					case LS_Endpoint:
						// Turn Line correction Off and symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

						// Stop flying
						stop_flying();
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_to_supply found L_Endpoint Supply\r\n");
						current_location = L_Supply;
						break;
					default:
						break;
				}
			}
			break;

		case L_Supply:
			// Correction is done and the drone is correctly orientated on top of the symbol
			if(receivedSignal(symbol_correction_done))
			{
				// Turn symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Supply starting Dock\r\n");
				//sendSignal(end_point_dock_start);
			}
			else if(receivedSignal(end_point_dock_complete))//dock process done.
			{
				// Start rotating 180 degrees
				rotate(1);
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - L_Supply Dock complete, rotating to find L_iSupply\r\n");
				destination = D_Delivery;
			}
			else if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_iEndpoint:
						rotate(0);
						emptySignalQueue(on_symbol);
						current_location = L_iSupply;
						// turn on Symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Supply found L_iSupply\r\n");
						break;
					default:
						break;
				}
			}
			break;

		case L_iSupply:
			// Correction is done and the drone is correctly oriented on top of the symbol
			if(receivedSignal(symbol_correction_done))
			{
				// Turn Symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

				// Fly forward
				start_flying();
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - To L_iSupply going from L_from_supply\r\n");
				current_location = L_From_supply;
			}
			break;

		case L_From_supply:
			if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_Line:
						// Turn Line correction On
						line_corr_switch = On;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Supply found LS_Line\r\n");
						break;
					case LS_iService:
						// Turn Line correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;

						// Check charging or standby is needed
						if (chargingneeded || go_standby)
						{
							// Stop to charge
							stop_flying();
							current_location = L_iService_dock;
							// Symbol correction On
							symb_corr_switch = On;
							if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Supply found LS_iService charging or standby needed\r\n");
						}
						else
						{
							// Keep flying past servicestation
							current_location = L_To_delivery;
							if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Supply found LS_iService keep flying to L_To_Delivery\r\n");

						}
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						break;
					default:
						break;
				}
			}
			// Charging needed signal
			else if(receivedSignal(charging_needed))
			{
				chargingneeded = 1;
			}
			// Console stop signal received, going into standby mode
			else if(receivedSignal(stop_drone))
			{
				go_standby = 1;
			}
			break;

		case L_To_delivery:
			if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_Line:
						// Turn Line correction On
						line_corr_switch = On;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_To_Delivery found LS_Line\r\n");

						break;
					case LS_Endpoint:
						// Turn Line correction Off and symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

						// Stop flying
						stop_flying();
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_To_Delivery found LS_Endpoint L_Delivery\r\n");
						current_location = L_Delivery;
					default:
						break;
				}
			}
			break;

		case L_Delivery:
			// Correction is done and the drone is correctly orientated on top of the symbol
			if(receivedSignal(symbol_correction_done))
			{
				// Turn symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Delivery starting Dock \r\n");
				//sendSignal(end_point_dock_start);
			}
			// Docking process done
			else if(receivedSignal(end_point_dock_complete))
			{
				// Start rotating 180 degrees
				rotate(1);
				destination = D_Supply;
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - L_Delivery Dock complete rotate to find L_iDelivery \r\n");
			}
			else if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_iEndpoint:
						rotate(0);
						emptySignalQueue(on_symbol);
						current_location = L_iDelivery;
						// Turn Symbol correction On
						line_corr_switch = Off;
						symb_corr_switch = On;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_Delivery found LS_iEndpoint L_iDelivery \r\n");
						break;
					default:
						break;
				}
			}
			break;

		case L_iDelivery:
			// Correction is done and the drone is correctly oriented on top of the symbol
			if(receivedSignal(symbol_correction_done))
			{
				// Turn Symbol correction Off
				line_corr_switch = Off;
				symb_corr_switch = Off;
				//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);

				// Fly forward
				start_flying();
				if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - At L_iDelivery fly to L_From_Delivery\r\n");
				current_location = L_From_delivery;
			}
			break;

		case L_From_delivery:
			if(receivedSignal(on_symbol, &current_symbol))
			{
				switch(current_symbol)
				{
					case LS_Line:
						// Turn Line correction On
						line_corr_switch = On;
						symb_corr_switch = Off;
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Delivery found LS_Line\r\n");
						break;
					case LS_Service:
						// Turn Line correction Off
						line_corr_switch = Off;
						symb_corr_switch = Off;

						if (chargingneeded || go_standby)
						{
							// Stop to charge
							stop_flying();
							if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Delivery found LS_Service charging needed\r\n");
							current_location = L_Service_dock;
							// Symbol correction On
							symb_corr_switch = On;
						}
						else
						{
							// Keep flying past servicestation
							current_location = L_To_supply;
							if (DEBUG && NAV_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Navigator - Operation L_From_Delivery found LS_Service keep flying to L_To_supply\r\n");
						}
						//sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
						break;
					default:
						break;
				}
			}
			// Charging needed signal
			else if(receivedSignal(charging_needed))
			{
				chargingneeded = 1;
			}
			// Console stop signal received, going into standby mode
			else if(receivedSignal(stop_drone))
			{
				go_standby = 1;
			}
			break;
		}
} //end navigator

