/*
 * Servicestation docking process
 *
 */

#include "definitions.h"

// Landing lightmode control value - LAND (1900) number 0 in array
#define LAND_VALUE 0

// Take-off lightmode control value - FM1 (1500) number 1 in array
#define LIFTOFF_VALUE 1

// Waiting period for landing and take-off process
#define WAIT_TIME 500

// Land on servicestation
static void land(void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	//signal with the correct information
	control_cmd = CC_Flightmode;
	control_value = LAND_VALUE;
	sendSignal(control_command, &control_cmd, &control_value);
}

// Take-off from servicestation
static char liftoff(void)
{
	static int control_value;
	static ControlcommandType control_cmd;
	static char order = 0;
	//signal with the correct information
	switch(order)
	{
		case 0:
			control_cmd =  CC_Yaw;
			control_value = 0;
			sendSignal(control_command, &control_cmd, &control_value);
			order++;
			break;
		case 1:
			control_cmd =  CC_Yaw;
			control_value = 5;
			sendSignal(control_command, &control_cmd, &control_value);
			order++;
			break;
		case 2:
			control_cmd =  CC_Throttle;
			control_value = 1;
			sendSignal(control_command, &control_cmd, &control_value);
			order++;
			break;
		case 3:
			control_cmd =  CC_Flightmode;
			control_value = 1;
			sendSignal(control_command, &control_cmd, &control_value);
			order = 0;
			return 1;
			break;
	}

	return 0;
}

void service_dock(void)
{
	static int init = 0;
	//static char stop_flag;
	static enum {waiting, landing, normal_charge, standby, lift_off} order;
	static ServiceType stype;
	static Timer t1;
	static char drone_start_seq = 0;
	static char ret;

	// Initialize
	if (!init)
	{
		order =  standby;
		init=1;
	}

	// Start sequence of events to land and dock onto the servicestation
	switch(order)
	{
		case waiting:
			if (receivedSignal(service_dock_start, &stype))
			{
				land();
				startTimer(WAIT_TIME, t1);
				order = landing;
			}
			break;

		case landing:
			if(timeout(t1))
			{
				switch(stype)
				{
					case S_Standby:
						order = standby;
						break;
					case S_Charge:
						order = normal_charge;
						break;
				}
			}
			break;

		case normal_charge:
			if(!drone_start_seq && receivedSignal(charging_done))
			{
				ret= liftoff();
				drone_start_seq = 1;

			}
			else if (drone_start_seq)
			{
				ret= liftoff();
				if (ret)
				{
					drone_start_seq = 1;
					order = lift_off;
					startTimer(WAIT_TIME, t1);
				}
			}
			break;
		case standby:
			// Start the drone - NOTE: might need to have it fiddle with battery_value in sensormonitor else
			// it might not be able to sense if battery is low again after a long standby period on the station
			if (!drone_start_seq && receivedSignal(start_drone))
			{
				ret= liftoff();
				drone_start_seq = 1;
			}
			else if (drone_start_seq)
			{
				ret= liftoff();
				if (ret)
				{
					drone_start_seq = 1;
					order = lift_off;
					startTimer(WAIT_TIME, t1);
					sendSignal(reset_batterycheck);
				}
			}
			break;
		case lift_off:
			if (timeout(t1))
			{
				sendSignal(service_dock_complete);
				order = waiting;
			}
			break;
	}
} // end service_dock
