/*
 * Symbol recognition process
 *
 */

#include "definitions.h"

// Spesific symbol buffers to make symbol rec flexible
#define ENDPOINT_X_BUFFER_VALUE 100
#define ENDPOINT_Y_BUFFER_LOW_VALUE 170
//#define ENDPOINT_Y_BUFFER_HIGH_VALUE 230
#define EDNPOINT_LONG_X_BUFFER_VALUE 20

#define SERVICE_Y_BUFFER_VALUE 20
#define SERVICE_Y_BUFFER_HIGH_VALUE 70
//#define SERVICE_X_BUFFER_LOW_VALUE 70
#define SERVICE_X_BUFFER_HIGH_VALUE 130

// Define center of IR camera coordinate system
#define XCENTER 511 // 1023/2
#define YCENTER 383 // 768/2

// Coordinate blob mapping
#define BLOB0 0
#define BLOB1 1
#define BLOB2 2
#define BLOB3 3

// Up to four points can be tracked
Blob temp_blob[4], blob[4];

// Perform distance comparison
char equal_space(int dist1, int dist2)
{
	if (abs(dist1)+30 >= abs(dist2) &&
			abs(dist1)-30 <= abs(dist2) )
	{
			return 1;
	} else {
		return 0;
	}
}

// Three points in a certain pattern is a line
char is_line_three()
{
	if (equal_space(blob[0].x-blob[1].x, blob[2].x-blob[1].x) &&
			equal_space(blob[2].y-blob[1].y, blob[1].y-blob[0].y))
	{
		return 1;
	} else {
		return 0;
	}
}

// Four points in a certain pattern is a line
char is_line_four()
{
	if (equal_space(blob[0].x-blob[1].x, blob[2].x-blob[3].x) &&
			equal_space(blob[3].y-blob[2].y, blob[1].y-blob[0].y))
	{
		return 1;
	} else {
		return 0;
	}
}

// Four points in a certain pattern with one tangent point double the distance out is a normal endpoint symbol
char is_normal_endpoint()
{
	static int average_y, average_x;
	static int low_x_blob, mid_x_blob, high_x_blob;

	average_x = ((blob[0].x + blob[1].x + blob[2].x + blob[3].x)/4);
	average_y = ((blob[0].y + blob[1].y + blob[2].y + blob[3].y)/4);

	// Pivot point, Find order of the 3 blobs at top
	if (blob[1].x > blob[2].x)
	{
		if(blob[2].x > blob[3]. x)
		{
			high_x_blob = 1;
			mid_x_blob = 2;
			low_x_blob = 3;
		}
		else
		{
			if (blob[1].x > blob[3].x)
			{
				high_x_blob = 1;
				mid_x_blob = 3;
				low_x_blob = 2;
			}
			else
			{
				high_x_blob = 3;
				mid_x_blob = 1;
				low_x_blob = 2;
			}
		}
	}
	else
	{
		if(blob[1].x > blob[3]. x)
		{
			high_x_blob = 2;
			mid_x_blob = 1;
			low_x_blob = 3;

		}
		else
		{
			if (blob[2].x > blob[3].x)
			{
				high_x_blob = 2;
				mid_x_blob = 3;
				low_x_blob = 1;
			}
			else
			{
				high_x_blob = 3;
				mid_x_blob = 2;
				low_x_blob = 1;
			}
		}
	}

	if (equal_space(blob[2].x-blob[1].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[1].y) &&
			blob[2].x <= blob[0].x &&
			blob[2].y >= blob[0].y &&
			equal_space(blob[3].x-blob[1].x, blob[2].y-blob[0].y) &&
			// decreasing blob x order
			blob[3].x >= blob[2].x &&
			blob[2].x >= blob[1].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal endpoint, return 1\r\n");
		return 1;
	} else if (equal_space(blob[2].x-blob[0].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[0].y) &&
			blob[1].x >= blob[2].x &&
			blob[1].y <= blob[2].y &&
			equal_space(blob[1].x-blob[2].x, blob[3].y-blob[0].y) &&
			// decreasing blob x order
			blob[3].x >= blob[2].x &&
			blob[2].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal endpoint, return 2\r\n");
		return 2;
	} else if (equal_space(blob[2].x-blob[1].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[1].y) &&
			blob[2].x >= blob[0].x &&
			blob[2].y >= blob[0].y &&
			equal_space(blob[1].x-blob[3].x, blob[2].y-blob[0].y) &&
			// decreasing blob x order
			blob[1].x >= blob[2].x &&
			blob[2].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal endpoint, return 3\r\n");
		return 3;
	} else if (equal_space(blob[2].x-blob[0].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[0].y) &&
			blob[1].x <= blob[2].x &&
			blob[1].y <= blob[2].y &&
			equal_space(blob[2].x-blob[1].x, blob[3].y-blob[0].y) &&
			// decreasing blob x order
			blob[0].x >= blob[2].x &&
			blob[2].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal endpoint, return 4\r\n");
		return 4;
	} else if ((blob[low_x_blob].y > average_y)&&(blob[mid_x_blob].y > average_y)&&
			(blob[high_x_blob].y > average_y) && (blob[0].y < (average_y -  ENDPOINT_Y_BUFFER_LOW_VALUE))&&
			((blob[low_x_blob].x < (average_x - ENDPOINT_X_BUFFER_VALUE)) &&(blob[high_x_blob].x > (average_x + ENDPOINT_X_BUFFER_VALUE))&&
			(blob[0].x <= (average_x + EDNPOINT_LONG_X_BUFFER_VALUE)) && (blob[0].x >= (average_x - EDNPOINT_LONG_X_BUFFER_VALUE)) &&
			(blob[mid_x_blob].x <= (average_x + EDNPOINT_LONG_X_BUFFER_VALUE)) && (blob[mid_x_blob].x >= (average_x - EDNPOINT_LONG_X_BUFFER_VALUE))))
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal endpoint, return 5 (center point)\r\n");
		return 5;
	} else {
		return 0;
	}
}

// Four points in a certain pattern with one point mirrored is an inverted endpoint symbol
char is_inv_endpoint()
{
	static int average_y, average_x;
	static int low_x_blob, mid_x_blob, high_x_blob;

	average_x = ((blob[0].x + blob[1].x + blob[2].x + blob[3].x)/4);
	average_y = ((blob[0].y + blob[1].y + blob[2].y + blob[3].y)/4);

	// Pivot point, find order of the 3 blobs at top
	if (blob[0].x > blob[1].x)
	{
		if(blob[1].x > blob[2]. x)
		{
			high_x_blob = 0;
			mid_x_blob = 1;
			low_x_blob = 2;
		}
		else
		{
			if (blob[0].x > blob[2].x)
			{
				high_x_blob = 0;
				mid_x_blob = 2;
				low_x_blob = 1;
			}
			else
			{
				high_x_blob = 2;
				mid_x_blob = 0;
				low_x_blob = 1;
			}
		}
	}
	else
	{
		if(blob[0].x > blob[2]. x)
		{
			high_x_blob = 1;
			mid_x_blob = 0;
			low_x_blob = 2;
		}
		else
		{
			if (blob[1].x > blob[2].x)
			{
				high_x_blob = 1;
				mid_x_blob = 2;
				low_x_blob = 0;
			}
			else
			{
				high_x_blob = 2;
				mid_x_blob = 1;
				low_x_blob = 0;
			}
		}
	}

	if (equal_space(blob[1].x-blob[0].x, blob[2].x-blob[1].x) &&
			equal_space(blob[2].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[1].x >= blob[3].x &&
			blob[1].y <= blob[3].y &&
			equal_space(blob[2].x-blob[0].x, blob[3].y-blob[1].y) &&
			// decreasing blob x order
			blob[2].x >= blob[1].x &&
			blob[1].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted endpoint, return 1\r\n");
		return 1;
	} else 	if (equal_space(blob[1].x-blob[0].x, blob[3].x-blob[1].x) &&
			equal_space(blob[3].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[2].x <= blob[1].x &&
			blob[2].y >= blob[1].y &&
			equal_space(blob[1].x-blob[2].x, blob[3].y-blob[0].y) &&
			// decreasing blob x order
			blob[3].x >= blob[1].x &&
			blob[1].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted endpoint, return 2\r\n");
		return 2;
	} else if (equal_space(blob[1].x-blob[0].x, blob[2].x-blob[1].x) &&
			equal_space(blob[2].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[1].x <= blob[3].x &&
			blob[1].y <= blob[3].y &&
			equal_space(blob[0].x-blob[2].x, blob[3].y-blob[1].y) &&
			// decreasing blob x order
			blob[0].x >= blob[1].x &&
			blob[1].x >= blob[2].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted endpoint, return 3\r\n");
		return 3;
	} else if (equal_space(blob[1].x-blob[0].x, blob[3].x-blob[1].x) &&
			equal_space(blob[3].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[2].x >= blob[1].x &&
			blob[2].y >= blob[1].y &&
			equal_space(blob[2].x-blob[1].x, blob[3].y-blob[0].y) &&
			// decreasing blob x order
			blob[0].x >= blob[1].x &&
			blob[1].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted endpoint, return 4\r\n");
		return 4;
	} else if ((blob[low_x_blob].y < average_y)&&(blob[mid_x_blob].y < average_y)&&
			(blob[high_x_blob].y < average_y) && (blob[3].y > (average_y + ENDPOINT_Y_BUFFER_LOW_VALUE))&&
			((blob[low_x_blob].x < (average_x - ENDPOINT_X_BUFFER_VALUE)) &&(blob[high_x_blob].x > (average_x + ENDPOINT_X_BUFFER_VALUE))&&
			(blob[3].x <= (average_x + EDNPOINT_LONG_X_BUFFER_VALUE)) && (blob[3].x >= (average_x -EDNPOINT_LONG_X_BUFFER_VALUE)) &&
			(blob[mid_x_blob].x <= (average_x + EDNPOINT_LONG_X_BUFFER_VALUE)) && (blob[mid_x_blob].x >= (average_x -EDNPOINT_LONG_X_BUFFER_VALUE))))
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted endpoint, return 5 (center point)\r\n");
		return 5;
	} else {
		return 0;
	}
}


// Four points in a certain pattern with one point tangent is a normal servicepoint symbol
char is_normal_service()
{
	static int average_y, average_x;
	static int mid_left_blob, mid_right_blob;

	average_x = ((blob[0].x + blob[1].x + blob[2].x + blob[3].x)/4);
	average_y = ((blob[0].y + blob[1].y + blob[2].y + blob[3].y)/4);

	// Pivot point, find left and right blob order
	if (blob[2].x > blob[1].x)
	{
		mid_left_blob = 2;
		mid_right_blob = 1;
	}
	else
	{
		mid_left_blob = 1;
		mid_right_blob = 2;
	}

	if (equal_space(blob[1].x-blob[0].x, blob[3].x-blob[1].x) &&
			equal_space(blob[3].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[2].x >= blob[1].x &&
			blob[2].y >= blob[1].y &&
			// decreasing blob x order
			blob[0].x >= blob[1].x &&
			blob[1].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal service, return 1\r\n");
		return 1;
	} else if (equal_space(blob[1].x-blob[0].x, blob[2].x-blob[1].x) &&
			equal_space(blob[2].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[3].x >= blob[1].x &&
			blob[3].y >= blob[1].y &&
			// decreasing blob x order
			blob[0].x >= blob[1].x &&
			blob[1].x >= blob[2].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal service, return 2\r\n");
		return 2;
	} else if (equal_space(blob[2].x-blob[0].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[0].y) &&
			blob[1].x >= blob[2].x &&
			blob[1].y <= blob[2].y&&
			//blob[0].x <= blob[1].x &&
			// decreasing blob x order
			blob[3].x >= blob[2].x &&
			blob[2].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal service, return 3\r\n");
		return 3;
	} else if (equal_space(blob[2].x-blob[1].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[1].y) &&
			blob[0].x >= blob[2].x &&
			blob[0].y <= blob[2].y &&
			//blob[0].x >= blob[1].x &&
			// decreasing blob x order
			blob[3].x >= blob[2].x &&
			blob[2].x >= blob[1].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal service, return 4\r\n");
		return 4;
	} else if ((blob[0].x < average_x) && (blob[3].x < average_x) &&
				(blob[mid_left_blob].x > average_x) && (blob[mid_left_blob].x < (average_x + SERVICE_X_BUFFER_HIGH_VALUE))&&
				(blob[mid_right_blob].x < average_x) &&	(blob[0].y < (average_y - SERVICE_Y_BUFFER_HIGH_VALUE)) &&
				(blob[3].y > (average_y + SERVICE_Y_BUFFER_HIGH_VALUE)) && (blob[mid_left_blob].y >= (average_y - SERVICE_Y_BUFFER_VALUE)) &&
				(blob[mid_left_blob].y <= (average_y + SERVICE_Y_BUFFER_VALUE)) && (blob[mid_right_blob].y >= (average_y - SERVICE_Y_BUFFER_VALUE)) &&
				(blob[mid_right_blob].y <= (average_y + SERVICE_Y_BUFFER_VALUE)))
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is normal service, return 5 (center point)\r\n");
		return 5;
	} else {
		return 0;
	}
}

// Four points in a certain pattern with one point mirrored is an inverted servicepoint symbol
char is_inverted_service()
{
	static int average_y, average_x;
	static int mid_left_blob, mid_right_blob;

	average_x = ((blob[0].x + blob[1].x + blob[2].x + blob[3].x)/4);
	average_y = ((blob[0].y + blob[1].y + blob[2].y + blob[3].y)/4);

	// Pivot point, find left and right blob order
	if (blob[2].x > blob[1].x)
	{
		mid_left_blob = 2;
		mid_right_blob = 1;
	}
	else
	{
		mid_left_blob = 1;
		mid_right_blob = 2;
	}

	if (equal_space(blob[1].x-blob[0].x, blob[3].x-blob[1].x) &&
			equal_space(blob[3].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[2].x <= blob[1].x &&
			blob[2].y >= blob[1].y &&
			// decreasing blob x order
			blob[3].x >= blob[1].x &&
			blob[1].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted service, return 1\r\n");
		return 1;
	} else if (equal_space(blob[1].x-blob[0].x, blob[2].x-blob[1].x) &&
			equal_space(blob[2].y-blob[1].y, blob[1].y-blob[0].y) &&
			blob[3].x <= blob[1].x &&
			blob[3].y >= blob[1].y &&
			// decreasing blob x order
			blob[2].x >= blob[1].x &&
			blob[1].x >= blob[0].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted service, return 2\r\n");
		return 2;
	} else if (equal_space(blob[2].x-blob[0].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[0].y) &&
			blob[1].x <= blob[2].x &&
			blob[1].y <= blob[2].y &&
			// decreasing blob x order
			blob[0].x >= blob[2].x &&
			blob[2].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted service, return 3\r\n");
		return 3;
	} else if (equal_space(blob[2].x-blob[1].x, blob[3].x-blob[2].x) &&
			equal_space(blob[3].y-blob[2].y, blob[2].y-blob[1].y) &&
			blob[0].x <= blob[2].x &&
			blob[0].y <= blob[2].y &&
			//blob[1].x <= blob[0].x &&
			// decreasing blob x order
			blob[1].x >= blob[2].x &&
			blob[2].x >= blob[3].x)
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted service, return 4\r\n");
		return 4;
	} else if ((blob[0].x > average_x) && (blob[3].x > average_x) &&
				(blob[mid_left_blob].x > average_x) && (blob[mid_right_blob].x < average_x) &&
				(blob[mid_right_blob].x > (average_x - SERVICE_X_BUFFER_HIGH_VALUE)) &&	(blob[0].y < (average_y - SERVICE_Y_BUFFER_HIGH_VALUE)) &&
				(blob[3].y > (average_y + SERVICE_Y_BUFFER_HIGH_VALUE)) && (blob[mid_left_blob].y >= (average_y - SERVICE_Y_BUFFER_VALUE)) &&
				(blob[mid_left_blob].y <= (average_y + SERVICE_Y_BUFFER_VALUE)) && (blob[mid_right_blob].y >= (average_y - SERVICE_Y_BUFFER_VALUE)) &&
				(blob[mid_right_blob].y <= (average_y+SERVICE_Y_BUFFER_VALUE)))
	{
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Is inverted service, return 5 (center point)\r\n");
		return 5;
	}else {
		return 0;
	}
}

// Calculate symbol correction values for x-axis, y-axis and angular offset
void symbol_correction_values(int value1, int value2)
{
	static double angle;
	static double x_offset;
	static double y_offset;

	angle = (float)(atan((blob[value1].x - blob[value2].x)/((float)(blob[value1].y - blob[value2].y))));
	x_offset =  (((blob[value1].x + blob[value2].x)/2) - (XCENTER));
	y_offset =  (((blob[value1].y + blob[value2].y)/2) - (YCENTER));

	sendSignal(symbol_correction_data, &angle, &x_offset, &y_offset);
	if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Symbol correction, angle: %d , x_offset: %d , y_offset: %d\r\n", (signed int)(angle*180/M_PI), (signed int)(x_offset), (signed int)(y_offset));
}

// Calculate line correction values for x-axis and angular offset
void line_correction_values(int value1, int value2)
{
	static double angle;
	static double x_offset;
	angle = (float)(atan((blob[value1].x - blob[value2].x)/((float)(blob[value1].y - blob[value2].y))));
	x_offset =  (((blob[value1].x + blob[value2].x)/(float)2) - (XCENTER));

	sendSignal(line_correction_data, &angle, &x_offset);
	if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Line correction, angle: %d , x_offset: %d\r\n", (signed int)(angle*180/M_PI), (signed int)(x_offset));
}

// Organize and sort blobs in order of 0-3 in respect to x and y coordinates 0,0 in
// upper right corner and down to up to 1024,768 in the bottom left corner
void blob_sorting(int variable)
{
	int score[4] = {(variable - 1), (variable - 1), (variable - 1), (variable - 1) };

	// Not sure it is needed, think they are already sorted
    for(int i = 0; i< variable; i++)
    {
    	for ( int j = 0; j <variable; j++)
    	{
    		if( i != j)
    		{
    			if ((temp_blob[i].y < (temp_blob[j].y)))
    			{
    				score[i]--;
    			}
    			else if((temp_blob[i].y >= (temp_blob[j].y)) && (temp_blob[i].y <= (temp_blob[j].y)))
    			{
    				if(temp_blob[i].x < temp_blob[j].x)
    				{
    					score[i]--;
    				}
    			}
    			else
    			{
    				// Nothing to do
    			}
    		}
    	}
    }

    for(int k=0; k<variable; k++)
    {
    	blob[score[k]] = temp_blob[k];
    }

    // Debug loop
    for(int k=0; k<variable; k++)
    {
    	if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Blob sorting done, score %d, blob #%d x %d y %d\r\n", score[k], k, blob[k].x, blob[k].y);
    }
}

// Find the number of valid blobs observed and received from IR camera
int blob_count_order()
{
	int amount = 0;

	for(int i = 0; i < 4; i++)
	{
		if ((blob[i].x != -1) && (blob[i].y != -1))
		{
			temp_blob[amount] = blob[i];
			amount++;
		}
	}
	return amount;
}

// Perform recognition of patterns to known logic symbols (three/four points -> line, four points -> endpoint,  four points -> servicepoint)
void symbol_recognizer(void)
{
	static int init = 0;
	static char blob_amount = 0;
	static LinesymbolType current_symbol, last_symbol;
	static char rectype;
	static SwitchType line_corr_switch, symb_corr_switch;

	// Initialize
	if (!init)
	{

		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - init");
		line_corr_switch = Off, symb_corr_switch = Off;
		init = 1;
	}

	if (receivedSignal(correction_switch, &line_corr_switch, &symb_corr_switch))
	{
		// this will then change the current variables into the ones in the signal
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Changed correction switch values, line corr %d, symb corr %d",line_corr_switch, symb_corr_switch);
	}
	// Receive data from IR camera
	else if (receivedSignal(blob_data, &blob[0].x, &blob[0].y, &blob[0].size, &blob[1].x, &blob[1].y, &blob[1].size, &blob[2].x, &blob[2].y, &blob[2].size, &blob[3].x, &blob[3].y, &blob[3].size))
	{
		// Find the number of blobs
		blob_amount = blob_count_order();
		if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Blob amount %d\r\n", blob_amount);

		// Rearrange the blobs in a predetermined order
		blob_sorting(blob_amount);

		// Perform pattern recognition depending on the number of blobs available
		switch (blob_amount)
		{
			case 0:
				// Stop
				break;
			case 1:
				// Keep flying
				break;
			case 2:
				// Keep flying
				break;
			case 3:
				// Checking y-distance between the dots, if they are equal within the buffer they are linear. Meaning line
				if(is_line_three())
				{
					current_symbol = LS_Line;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, &current_symbol);
					}

					if (line_corr_switch) {
						line_correction_values(BLOB2,BLOB0);
					}

					last_symbol = current_symbol;

					if (DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Line observed, amount of blobs %d\r\n", blob_amount);

				}
				else
				{
					// Keep Flying
				}

				break;
			case 4:
				// Checking y-distance between the dots, if they are equal within the buffer they are linear. Meaning line
				if(is_line_four())
				{
					current_symbol = LS_Line;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, &current_symbol);
					}

					if (line_corr_switch) {
						line_correction_values(BLOB3,BLOB0);
					}

					last_symbol = current_symbol;
					if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Line observed, amount of blobs %d\r\n", blob_amount);
					//usartPrint(0, "\n\rCorrection, angle: %d , x_offset: %d \r\n", (signed int)(angle*180/M_PI), (signed int)(x_offset));
				}
				// Checking for endpoint normal
				else if ((rectype = is_normal_endpoint()) && rectype)
				{
					current_symbol = LS_Endpoint;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, & current_symbol);
					}

					if (symb_corr_switch)
					{
						switch (rectype)
						{
							case 1:
								symbol_correction_values (BLOB3,BLOB1);
								break;
							case 2:
								symbol_correction_values (BLOB2,BLOB1);
								break;
							case 3:
								symbol_correction_values (BLOB3,BLOB1);
								break;
							case 4:
								symbol_correction_values (BLOB2,BLOB1);
								break;
							case 5:
								symbol_correction_values (BLOB3,BLOB1);
								break;

						}
					}

					last_symbol = current_symbol;
					if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Normal endpoint observed, amount of blobs %d\r\n", blob_amount);

				}
				// Checking for endpoint inverted
				else if ((rectype = is_inv_endpoint()) && rectype)
				{
					current_symbol = LS_iEndpoint;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, &current_symbol);
					}

					if (symb_corr_switch)
					{
						switch (rectype)
						{
							case 1:
								symbol_correction_values (BLOB2,BLOB0);
								break;
							case 2:
								symbol_correction_values (BLOB2,BLOB1);
								break;
							case 3:
								symbol_correction_values (BLOB2,BLOB0);
								break;
							case 4:
								symbol_correction_values (BLOB2,BLOB1);
								break;
							case 5:
								symbol_correction_values (BLOB2,BLOB0);
								break;
						}
					}

					last_symbol = current_symbol;
					if (DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Inverted endpoint observed, amount of blobs %d\r\n", blob_amount);

				}
				// Checking for normal service point
				else if ((rectype = is_normal_service()) && rectype)
				{
					current_symbol = LS_Service;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, &current_symbol);
					}

					if (symb_corr_switch)
					{
						switch (rectype)
						{
							case 1:
								symbol_correction_values (BLOB3,BLOB0);
								break;
							case 2:
								symbol_correction_values (BLOB2,BLOB0);
								break;
							case 3:
								symbol_correction_values (BLOB3,BLOB0);
								break;
							case 4:
								symbol_correction_values (BLOB3,BLOB1);
								break;
							case 5:
								symbol_correction_values (BLOB3,BLOB0);
								break;
						}
					}

					last_symbol = current_symbol;

					if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Normal servicepoint, amount of blobs %d\r\n", blob_amount);
				}
				// Check for inverted servicepoint
				else if ((rectype = is_inverted_service()) && rectype)
				{
					current_symbol = LS_iService;

					if (current_symbol != last_symbol)
					{
						sendSignal(on_symbol, &current_symbol);
					}

					if (symb_corr_switch)
					{
						switch (rectype)
						{
							case 1:
								symbol_correction_values (BLOB3,BLOB0);
								break;
							case 2:
								symbol_correction_values (BLOB2,BLOB0);
								break;
							case 3:
								symbol_correction_values (BLOB3,BLOB0);
								break;
							case 4:
								symbol_correction_values (BLOB3,BLOB1);
								break;
							case 5:
								symbol_correction_values (BLOB3,BLOB0);
								break;
						}
					}

					last_symbol = current_symbol;
					if (DEBUG && SYM_REC_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Symbol rec - Inverted servicepoint, amount of blobs %d\r\n", blob_amount);
				}
				else
				{
					// Keep flying and do no correction
				}
				break;

		} // end switch
	} // end if
} // end symbol_recognizer
