/* 
 * Autonomous system - testing processes
 *
 */

#include "definitions.h"
#include <arduino/usarts.h>

#define MAX_CMD 20
#define MAX_MSG 120

#define SERIAL_OUTBOUND 0
#define SERIAL_INBOUND 3 // usb 0 xbee 3
#define BAUD_RATE 57600
#define MAX_CMD 20
#define MAX_MSG 120

/***********************************************************/
/***********************************************************/
/* ircamera.c test*/

void ircamera_test(void)
{
	static Timer t1;
	static int init = 0;
	//int blobs[4];
	//Blob blob1;

	if (!init)
	{
		usartInstall(DEBUG_PORT,57600); // ut
		usartPrint(DEBUG_PORT,"\r\nStarting terminal...");

	    // Initilize i2c and sensor caps/integration values
        usartPrint(0,"i2c init");
	    i2c_init();

		t1 = newTimer();
		startTimer(10, t1);
		init = 1;
	}

	if (timeout(t1))
	{
		startTimer(100, t1);
		//usartPrint(DEBUG_PORT,"Getting colors...");
		// remove static in irkamera to get access
		//ircamera_getdata(blobs);

		//blob1 = blobs[0];
		//usartPrint(DEBUG_PORT, "Blob1: x %d y %d size %d\r\n", blob1.x,blob1.y,blob1.size);
		//usartPrint(DEBUG_PORT," Red %d Green %d Blue %d", colors[0],colors[1],colors[2]);

	}
}

void control_test(void)
{

	/*		mode  		number		pin		00		01		02		03		04		05		06		07		08		09		10
	 *
	 * 		Pitch		0			11		1410,	1440, 	1470, 	1485, 	1495, 	1500, 	1505, 	1515, 	1530, 	1560, 	1590
	 * 		Roll		1			12		1410,	1440, 	1470, 	1485, 	1495, 	1500, 	1505, 	1515, 	1530, 	1560, 	1590
	 * 		Throttle	2			13		0, 		1500,  	1600
	 * 		Flightmode	3			6		1900, 	1500, 	1100
	 * 		Rudder		4			7		1100, 	1300, 	1350, 	1400, 	1450, 	1500, 	1550, 	1600, 	1650, 	1700,	1900
	 *
	 */

	static int init =0;
	static char a, c[4];

	static int mode;
	static int pulsewidth;
	static int i = 0;

	if (!init)
	{
		usartInstall(DEBUG_PORT,57600); // ut
		usartPrint(DEBUG_PORT,"Starting control testing...\r\n");
		init=1;
	}

	if (usartDataIsPresent(DEBUG_PORT))
	{
		usartPrint(DEBUG_PORT,"Receiving command...\r\n");
		memset(c, '\0', 4);
		do
		{
			a = usartRead(DEBUG_PORT);
			c[i]=a;

			if (i < (3))
				i++;
		} while (a != '\n');

		mode = (c[0]- '0');
		pulsewidth = ((c[1]- '0')*10 + (c[2]- '0'));
		i = 0;
		usartPrint(DEBUG_PORT,"Received command mode: %d value: %d\r\n", mode, pulsewidth);

		sendSignal(control_command, &mode, &pulsewidth);
	}

}


void flight_test_control_commands(void)
{
	static int init = 0;

	static int i = 0;
	static char c;
    static char a[MAX_CMD];

    static int code = 0;
    static int type;
    static int value = 0;
    static ControlcommandType control_cmds = 0;
    static SwitchType line_corr_switch, symb_corr_switch;

	if (!init)
	{
		//usartInstall(SERIAL_OUTBOUND, BAUD_RATE); // Outbound from robot (On-board Arduino)
		usartInstall(DEBUG_PORT, BAUD_RATE); // Inbound from computer (Xbee)
		//usartPrint(DEBUG_PORT, "Communicator init");
		if (INIT_DEBUG) usartPrint(DEBUG_PORT, "DEBUG: Communicator - init\r\n");
		init = 1;
	}

	// Receive commands from computer over Xbee link
	if (usartDataIsPresent(DEBUG_PORT))
	{
		memset(a, '\0', MAX_CMD);
		do
		{
			c = usartRead(DEBUG_PORT);

			a[i] = c;
			if (i < 4)
				i++;
		} while (c != '\n');

		//usartPrint(DEBUG_PORT, a);
		if (a[0] == 'x')
		{
			code = 'x';
			type = (a[1]-'0');
			value = (a[2]-'0');
		}
		else
		{
		code = (a[0]- '0');
		value = ((a[1]-'0')*10 + (a[2]-'0'));
		}
	    i = 0;

	    switch (code)
	    {
	    	case 0:
	    		control_cmds =  CC_Pitch;
	    		usartPrint(DEBUG_PORT, "Pitch code %d value %d\r\n", code, value);
	    		sendSignal(control_command, &control_cmds, &value);
	    		break;
	    	case 1:
	    		control_cmds =  CC_Roll;
	    		usartPrint(DEBUG_PORT, "Roll code %d value %d\r\n", code, value);
	    		sendSignal(control_command, &control_cmds, &value);
	    		break;
	    	case 2:
	    		control_cmds =  CC_Throttle;
	    		usartPrint(DEBUG_PORT, "Throttle code %d value %d\r\n", code, value);
	    		sendSignal(control_command, &control_cmds, &value);
	    		break;
	    	case 3:
	    		control_cmds =  CC_Flightmode;
	    		usartPrint(DEBUG_PORT, "Flight mode code %d value %d\r\n", code, value);
	    		sendSignal(control_command, &control_cmds, &value);
	    		break;
	    	case 4:
	    		control_cmds =  CC_Yaw;
	    		usartPrint(DEBUG_PORT, "Yaw code %d value %d\r\n", code, value);
	    		sendSignal(control_command, &control_cmds, &value);
	    		break;
	    	case 5:
	    		line_corr_switch = value;
	    		symb_corr_switch = Off;
	    		usartPrint(DEBUG_PORT, "Symbol correction off, line correction value %d\r\n", value);
	    		sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
	    		break;
	    	case 6:
	    		line_corr_switch = Off;
	    		symb_corr_switch = value;
	    		usartPrint(DEBUG_PORT, "Line correction off, symbol correction value %d\r\n", value);
	    		sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
	    		break;
	    	case 'x':
				debug_set(type, value);
	    		break;
	    	default:
	    		break;
	    }
	}
}

void navigator_test_cmds(void)
{
	static int init = 0;
	//static char status_message[MAX_MSG];
	static int i = 0;
	static char c;
    static char a[MAX_CMD];

    static int code = 0;
    static int value = 0;
    static int type = 0;


	if (!init)
	{
		//usartInstall(SERIAL_OUTBOUND, BAUD_RATE); // Outbound from robot (On-board Arduino)
		usartInstall(DEBUG_PORT, BAUD_RATE); // Inbound from computer (Xbee)
		//usartPrint(DEBUG_PORT, "Communicator init");
		if (INIT_DEBUG) usartPrint(DEBUG_PORT, "DEBUG: Communicator - init\r\n");
		init = 1;
	}

	// Receive commands from computer over Xbee link
	if (usartDataIsPresent(DEBUG_PORT))
	{
		memset(a, '\0', MAX_CMD);
		do
		{
			c = usartRead(DEBUG_PORT);

			a[i] = c;
			if (i < 15)
				i++;
		} while (c != '\n');

		//usartPrint(DEBUG_PORT, a);
		if( a[0] != 'x')
		{
			code = (a[0]- '0');
		}
		else
		{
		code = a[0];
		type = (a[1]-'0');
		value = (a[2]-'0');
		}
	    i = 0;

	    switch (code)
	    {

	    	case 0:
	    		sendSignal(symbol_correction_done);
	    		if (NAV_DEBUG) usartPrint(DEBUG_PORT,"Test CMD : Signal symbol_correction_done sent \r\n");
	    		break;
	    	case 1:
	    		sendSignal(end_point_dock_complete);
	    		if (NAV_DEBUG) usartPrint(DEBUG_PORT,"Test CMD : Signal end_point_dock_complete sent \r\n");
	    		break;
	    	case 2:
	    		sendSignal(service_dock_complete);
	    		if (NAV_DEBUG) usartPrint(DEBUG_PORT,"Test CMD : Signal service_dock_complete sent \r\n");
	    		break;
	    	case 3:
	    		sendSignal(charging_needed);
	    		if (NAV_DEBUG) usartPrint(DEBUG_PORT,"Test CMD : Signal charging_needed sent \r\n");
	    		break;
	    	case 'x':
	    		debug_set(type, value);
	    		break;
	    	/*case 4:
	    		sendSignal(charging_done);
	    		if (NAV_DEBUG) usartPrint(DEBUG_PORT,"Test CMD : Signal end_point_dock_complete sent \r\n");
	    		break;
	    	case 5:
	    		line_corr_switch = value;
	    		symb_corr_switch = off;
	    		usartPrint(DEBUG_PORT, "Symbol correction off, line correction value %d\r\n", value);
	    		sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
	    		break;
	    	case 6:
	    		line_corr_switch = off;
	    		symb_corr_switch = value;
	    		usartPrint(DEBUG_PORT, "Line correction off, symbol correction value %d\r\n", value);
	    		sendSignal(correction_switch, &line_corr_switch, &symb_corr_switch);
	    		break; */
	    	default:
	    		break;
	    }
	}
}

void sensor_test_cmds(void)
{
	static int init = 0;
	//static char status_message[MAX_MSG];
	static int i = 0;
	static char c;
    static char a[MAX_CMD];

    static int code = 0;
    static int value = 0;
    static int type = 0;


	if (!init)
	{
		//usartInstall(SERIAL_OUTBOUND, BAUD_RATE); // Outbound from robot (On-board Arduino)
		usartInstall(DEBUG_PORT, BAUD_RATE); // Inbound from computer (Xbee)
		//usartPrint(DEBUG_PORT, "Communicator init");
		if (INIT_DEBUG) usartPrint(DEBUG_PORT, "DEBUG: Communicator - init\r\n");
		init = 1;
	}

	// Receive commands from computer over Xbee link
	if (usartDataIsPresent(DEBUG_PORT))
	{
		memset(a, '\0', MAX_CMD);
		do
		{
			c = usartRead(DEBUG_PORT);

			a[i] = c;
			if (i < 4)
				i++;
		} while (c != '\n');

		//usartPrint(DEBUG_PORT, a);
		code = a[0];
		type = (a[1]-'0');
		value = (a[2]-'0');

	    i = 0;

	    switch (code)
	    {
	     	case 'x':
	    		debug_set(type, value);
	    		break;
	     	default:
	     		usartPrint(DEBUG_PORT, "DEBUG: not viable command\r\n");
	    		break;
	    }
	}
}



void terminal_test(void)
{
	static char init = 0;

	if (!init)
	{
		//usartInstall(SERIAL_OUTBOUND, BAUD_RATE); // Outbound from robot (On-board Arduino)
		usartInstall(3, 57600); // Inbound from computer (Xbee)
		//usartPrint(DEBUG_PORT, "Communicator init");
		usartPrint(3,"Test fra drone!");
		init = 1;
	}
}

/***********************************************************/
/***********************************************************/

