/*
 * Belt process
 *
 */

#include "definitions.h"

// Number of belt chairs that have to pass in order to move object to the top
#define CHAIRS_BEFORE_STOP 4

// Waiting period before engaging the belt system
#define WAIT_TIME_SMALL 500

void belt (void)
{
	static int init = 0;
	static enum {stopped, running} belt_situation;
	static char chair_count= 0;
	static char object_on_shelf = 0;
	static Timer t1;

	// Initialize
	if(!init)
	{
		//if (DEBUG && INIT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - init\r\n");
		t1= newTimer_328();
		belt_situation = running;
		sendSignal(belt_start);

		init = 1;
	}

	// Sequence of events to keep the elevator moving
	switch(belt_situation)
	{
		case running:
			if(receivedSignal(object_arrived_objectshelf_lift))
			{
				//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Running - object arrived shelf\r\n");
				chair_count = (CHAIRS_BEFORE_STOP - 1);
				object_on_shelf = 1;

			}
			else if(!object_on_shelf && receivedSignal(object_elevator))
			{
				//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Running - No object on shelf, new object in elevator\r\n");
				chair_count = 0;
			}
			else if(receivedSignal(elevator_count))
			{

				chair_count++;
				if (chair_count == CHAIRS_BEFORE_STOP)
				{
					//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Running - Elevator stop\r\n");
					sendSignal(belt_stop);
					chair_count = 0;
					belt_situation = stopped;
				}
				else
				{
					//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Running - Elevator count +1\r\n");
				}
			}

			break;

		case stopped:
			if (object_on_shelf && receivedSignal(object_left_objectshelf))
			{
				//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Stopped - Object left shelf\r\n");
				startTimer_328(WAIT_TIME_SMALL, t1);
			}
			else if(!object_on_shelf && receivedSignal(object_elevator))
			{
				//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Stopped - No object on shelf, new object in elevator\r\n");
				startTimer_328(WAIT_TIME_SMALL, t1);
			}
			else if (timeout_328(t1))
			{
				//if (DEBUG && BELT_DEBUG) usartWriteString_328(DEBUG_PORT, "B - Stopped - Start elevator\r\n");
				sendSignal(belt_start);
				object_on_shelf = 0;
				belt_situation = running;
			}
			break;
	}
}
