/*
 * Magnet process
 *
 */

#include "definitions.h"

// Pin number to activate magnet
#define MAGNET_PIN 7

// Waiting periods between state changes
#define WAIT_TIME_SMALL 500

// Activate electromagnet
static void magnet_on(void)
{
	//if (DEBUG && MAGNET_DEBUG) usartPrint_328(DEBUG_PORT, "M - Magnet on\r\n");
	digitalWrite_328(MAGNET_PIN, HIGH);
}

// Deactivate electromagnet
static void magnet_off(void)
{
	//if (DEBUG && MAGNET_DEBUG) usartPrint_328(DEBUG_PORT, "M - Magnet off\r\n");
	digitalWrite_328(MAGNET_PIN, LOW);
}

void magnet(void)
{
	static int init = 0;
	static enum { moving, inner_endpoint, outer_endpoint} current_position;
	static char order;
	static Timer t1;
	static char obj_magnet;

	// Initialize
	if (!init)
	{
		//if (DEBUG && INIT_DEBUG) usartWriteString_328(DEBUG_PORT, "M - init\r\n");
		pinMode_328(MAGNET_PIN, OUTPUT);
		// Default start configuration: magnet off, position outer
		magnet_off();
		obj_magnet = 0;
		sendSignal(magnet_out);
		current_position = moving;
		order = 1;
		t1 = newTimer_328();

		init =1;
	}

	// Sequence of events to keep the magnet moving back and forth
	switch(current_position)
	{
		case moving:
			if (receivedSignal(magnet_inner_endpoint))
			{
				//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Moving - Received inner endpoint\r\n");
				sendSignal(magnet_stop);
				order = 1;
				current_position = inner_endpoint;
				startTimer_328(WAIT_TIME_SMALL, t1);
			}
			else if (receivedSignal(magnet_outer_endpoint))
			{
				sendSignal(magnet_stop);
				if (obj_magnet)
				{
					//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Moving - Received outer endpoint, object on magnet\r\n");
					order = 1;
				}
				else
				{
					//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Moving - Received outer endpoint, no object on magnet\r\n");
					order = 5;
				}
				current_position = outer_endpoint;
				//emptySignalQueue(no_object_on_magnet);
			}
			break;

		case inner_endpoint:
			switch(order)
			{
				case 1:
					if(timeout_328(t1))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Inner position - 1/3 Activate magnet\r\n");
						magnet_on();

						//for sensor at the outer point
						order = 3;
						startTimer_328(WAIT_TIME_SMALL, t1);
						obj_magnet = 1;
						//for sensor on the magnet.
						//order = 2;
					}
					break;
				case 2:
					if (receivedSignal(object_on_magnet))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Inner position - 2/3 Object attached to magnet\r\n");
						order = 3;
						startTimer_328(WAIT_TIME_SMALL, t1);
						obj_magnet = 1;
					}
					break;
				case 3:
					if (timeout_328(t1))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Inner position - 3/3 Moving to outer\r\n");
						sendSignal(magnet_out);
						order = 1;
						current_position = moving;
					}
					break;
				default:
					break;
			}
			break;
		case outer_endpoint:

			switch(order)
			{
				case 1:
					if (receivedSignal(drone_docked))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Outer - 1/5 Drone docked\r\n");
						startTimer_328(WAIT_TIME_SMALL, t1);
						order= 2;
					}
					break;

				case 2:
					if (timeout_328(t1))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Outer - 2/5 Releasing object\r\n");
						magnet_off();
						order= 3;
					}
					break;

				case 3:
					if (receivedSignal(no_object_on_magnet))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Outer - 3/5 Object on magnet\r\n");
						startTimer_328(WAIT_TIME_SMALL,t1);
						order = 4;
						obj_magnet = 0;

					}
					break;
				case 4:
					if (timeout_328(t1))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Outer - 4/5 Waiting on new object\r\n");
						order = 5;
					}
					break;

				case 5:
					if (receivedSignal(object_arrived_objectshelf_magnet))
					{
						//if (DEBUG && MAGNET_DEBUG) usartWriteString_328(DEBUG_PORT, "M - Outer - 5/5 New object on shelf\r\n");
						sendSignal(magnet_in);
						order = 1;
						current_position = moving;
					}
					break;
			}
			break;
	}
}
