/*
 * Arduino ATmega328 pin layout
 *
 */

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/interrupt.h>

static const unsigned char PROGMEM pin_to_bit_PGM[] =
{
	0	, // PD 0 ** 0 ** USART0_TX
	1	, // PD 1 ** 1 ** USART0_RX
	2	, // PD 2 ** 2 ** D2
	3	, // PD 3 ** 3 ** D3
	4	, // PD 4 ** 4 ** D4
	5	, // PD 5 ** 5 ** D5
	6	, // PD 6 ** 7 ** D6
	7	, // PD 7 ** 7 ** D7
	0	, // PB 0 ** 8 ** D8
	1	, // PB 1 ** 9 ** D9
	2	, // PB 2 ** 10 ** D10
	3	, // PB 3 ** 11 ** D11 / MOSI
	4	, // PB 4 ** 12 ** D12 / MISO
	5	, // PB 5 ** 13 ** D13 / SCK
	0	, // PC 0 ** 14 ** A0
	1	, // PC 1 ** 15 ** A1
	2	, // PC 2 ** 16	** A2
	3	, // PC 3 ** 17 ** A3
	4	, // PC 4 ** 18	** A4 / SDA
	5	, // PC 5 ** 19 ** A5 / SCL
	
	
};

static const unsigned char * PROGMEM pin_to_PORT_PGM[] =
{

	&PORTD	, // PD 0 ** 0 ** USART0_TX
	&PORTD	, // PD 1 ** 1 ** USART0_RX
	&PORTD	, // PD 2 ** 2 ** D2
	&PORTD	, // PD 3 ** 3 ** D3
	&PORTD	, // PD 4 ** 4 ** D4
	&PORTD	, // PD 5 ** 5 ** D5
	&PORTD	, // PD 6 ** 7 ** D6
	&PORTD	, // PD 7 ** 7 ** D7
	&PORTB	, // PB 0 ** 8 ** D8
	&PORTB	, // PB 1 ** 9 ** D9
	&PORTB	, // PB 2 ** 10 ** D10
	&PORTB	, // PB 3 ** 11 ** D11 / MOSI
	&PORTB	, // PB 4 ** 12 ** D12 / MISO
	&PORTB	, // PB 5 ** 13 ** D13 / SCK
	&PORTC	, // PC 0 ** 14 ** A0
	&PORTC	, // PC 1 ** 15 ** A1
	&PORTC	, // PC 2 ** 16	** A2
	&PORTC	, // PC 3 ** 17 ** A3
	&PORTC	, // PC 4 ** 18	** A4 / SDA
	&PORTC	, // PC 5 ** 19 ** A5 / SCL
	
};

static const unsigned char * PROGMEM pin_to_PIN_PGM[] =
{
	&PIND	, // PD 0 ** 0 ** USART0_TX
	&PIND	, // PD 1 ** 1 ** USART0_RX
	&PIND	, // PD 2 ** 2 ** D2
	&PIND	, // PD 3 ** 3 ** D3
	&PIND	, // PD 4 ** 4 ** D4
	&PIND	, // PD 5 ** 5 ** D5
	&PIND	, // PD 6 ** 7 ** D6
	&PIND	, // PD 7 ** 7 ** D7
	&PINB	, // PB 0 ** 8 ** D8
	&PINB	, // PB 1 ** 9 ** D9
	&PINB	, // PB 2 ** 10 ** D10
	&PINB	, // PB 3 ** 11 ** D11 / MOSI
	&PINB	, // PB 4 ** 12 ** D12 / MISO
	&PINB	, // PB 5 ** 13 ** D13 / SCK
	&PINC	, // PC 0 ** 14 ** A0
	&PINC	, // PC 1 ** 15 ** A1
	&PINC	, // PC 2 ** 16	** A2
	&PINC	, // PC 3 ** 17 ** A3
	&PINC	, // PC 4 ** 18	** A4 / SDA
	&PINC	, // PC 5 ** 19 ** A5 / SCL
	
};

static const unsigned char * PROGMEM pin_to_DDR_PGM[] =
{
	&DDRD	, // PD 0 ** 0 ** USART0_TX
	&DDRD	, // PD 1 ** 1 ** USART0_RX
	&DDRD	, // PD 2 ** 2 ** D2
	&DDRD	, // PD 3 ** 3 ** D3
	&DDRD	, // PD 4 ** 4 ** D4
	&DDRD	, // PD 5 ** 5 ** D5
	&DDRD	, // PD 6 ** 7 ** D6
	&DDRD	, // PD 7 ** 7 ** D7
	&DDRB	, // PB 0 ** 8 ** D8
	&DDRB	, // PB 1 ** 9 ** D9
	&DDRB	, // PB 2 ** 10 ** D10
	&DDRB	, // PB 3 ** 11 ** D11 / MOSI
	&DDRB	, // PB 4 ** 12 ** D12 / MISO
	&DDRB	, // PB 5 ** 13 ** D13 / SCK
	&DDRC	, // PC 0 ** 14 ** A0
	&DDRC	, // PC 1 ** 15 ** A1
	&DDRC	, // PC 2 ** 16	** A2
	&DDRC	, // PC 3 ** 17 ** A3
	&DDRC	, // PC 4 ** 18	** A4 / SDA
	&DDRC	, // PC 5 ** 19 ** A5 / SCL

};

#define pin_to_PORT(P)((volatile unsigned char *)( pgm_read_word( pin_to_PORT_PGM + (P))) )
#define pin_to_PIN(P) ((volatile unsigned char *)( pgm_read_word( pin_to_PIN_PGM + (P))) )
#define pin_to_DDR(P) ((volatile unsigned char *)( pgm_read_word( pin_to_DDR_PGM + (P))) )
#define pin_to_bit(P) ((volatile unsigned char)( pgm_read_byte( pin_to_bit_PGM + (P))) )

void pinMode_328(unsigned char pin, unsigned char mode)
{
	volatile unsigned char * reg = pin_to_DDR(pin);
	unsigned char bit = pin_to_bit(pin);

	unsigned char oldSreg = SREG;
	cli();
	if (mode == 0)
		*reg &= ~(1<<bit);
	else
		*reg |= (1<<bit);
	SREG = oldSreg;
}

void digitalWrite_328(unsigned char pin, unsigned char val)
{

	volatile unsigned char * reg = pin_to_PORT(pin);
	unsigned char bit = pin_to_bit(pin);

	unsigned char oldSreg = SREG;
	cli();
	if (val == 0)
		*reg &= ~(1<<bit);
	else
		*reg |= (1<<bit);
	SREG = oldSreg;
}

int digitalRead_328(unsigned char pin)
{
	volatile unsigned char * reg = pin_to_PIN(pin);
	unsigned char bit = pin_to_bit(pin);

	unsigned char oldSreg = SREG;
	cli();
	int val = (*reg & (1<<bit)) != 0;
	SREG = oldSreg;
	return val;
}

