/* ---------------------------------------------------------

 Arduino ATmega328 timer

 Denne modulen eksporterer prosedyrer for  lage en tidsmler
 (Timer) i ATMega328.

 I tillegg ekporterer den prosedyrer for  mle tiden
 siden systemstart, og enkle "dumme" delay-funksjoner.

 Modulen benytter TIMER1 i ATMega328

--------------------------------------------------------- */

#ifndef TIMER_328_H
#define TIMER_328_H

typedef struct timer
{
    struct timer * next;
    volatile unsigned long counter;
    volatile int expired;
}
timer_type, * Timer;

/*
  -	Datatypen "Timer" identifiserer en tidsmler.
*/

Timer newTimer_328(void);
/*
  - Kreerer en tidsmler.

	Tidsmleren m deklareres fr den kan kreeres:
		Timer tid_inne;
		tid_inne = newTimer_328();
*/

void deleteTimer_328(Timer t);
/*
  -	Destruerer en tidsmler og frigjr allokert minne.
*/

void startTimer_328(unsigned long millisecs, Timer t);
/*
  -	Starter den spesifiserte tidsmleren.

 	Tidsmleren utlper etter det spesifiserte antall
 	millisekunder.

 	For  starte tidsmleren "tid_inne" til  utlpe
  	etter 1 sekund:
	startTimer_328(1000, tid_inne);
*/

int timeout_328(Timer t);
/*
  -	Returenerer true dersom tidsmleren er utlpt, false ellers.

	Benyttes typisk slik:

	if (timeout_328(tid_inne))
		<gjr det som gjres skal>
*/


void startTiming_328();
/*
  - Initialiserer systemet og starter tidsmlingen.

*/

unsigned long long timeSinceStart_328(); //Returnerer 64 bit integer!
/*
  - Returnerer antall mikrosekunder siden systemstart.

 */

void delay_328(unsigned long milliseconds);
/*
  - Vent i det spesifiserte antall millisekunder.

*/

void delayMicroseconds_328(unsigned int microseconds);
/*
  - Vent i det spesifiserte antall miikrosekuner.

  Maksimal ventetid er 4096 mikrosekunder!

*/


#endif

