/*
 * Service process
 *
 * INFO: currently only works in certain orientation due to feedback from 12V to the chargers.
 * Will most likely be sorted before presentation, using relee's to physical disconnect wires.
 */

#include "definitions.h"

#define WAIT_TIME 250 // time between connecting vss and connecting charging pins

//Vss on, and led pins
static unsigned char vss_pin_no[4] = { 9, 7, 5, 3 };

//Relee control pins
static unsigned char relee_pins[2] = {11, 10};

//Charger on pad led pins
static unsigned char charge_led_pin_no[4] = { 8, 6, 4, 2 };

//
static unsigned char charger_on_pin = 12;




static void set_vss_and_leds(char pad_no_12_volt)
{
	//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - VSS - loc %d, pin %d til %d\r\n"
	//		,loc_pin_no, vss_pin_no[loc_pin_no], !value);
	for(int i = 0; i < 4; i++)
	{
		// if higher than 3 then set all low
		if (pad_no_12_volt > 3)
		{
			digitalWrite_328(vss_pin_no[i], LOW);
			digitalWrite_328(charge_led_pin_no[i], LOW);
		}
		// if 12 volt pad number equals i, then set  12 volt pin high, and chargerled low
		else if(i == pad_no_12_volt)
		{
			digitalWrite_328(vss_pin_no[i], HIGH);
			digitalWrite_328(charge_led_pin_no[i], LOW);
		}
		// else set charge led high and 12 volt pin low
		else
		{
		digitalWrite_328(vss_pin_no[i], LOW);
		digitalWrite_328(charge_led_pin_no[i], HIGH);
		}
	}
}


static void set_charge(char value)
{
	switch(value)
	{
		case 0:
			digitalWrite_328(charger_on_pin, LOW);
			break;
		case 1:
			digitalWrite_328(charger_on_pin, HIGH);
			break;

	}
}

static void set_relee(char pad_no_12_volt)
{
	switch(pad_no_12_volt)
	{
		case 0:
			digitalWrite_328(relee_pins[0], HIGH);
			digitalWrite_328(relee_pins[1], LOW);
			break;
		case 1:
			digitalWrite_328(relee_pins[0], HIGH);
			digitalWrite_328(relee_pins[1], HIGH);
			break;
		case 2:
			digitalWrite_328(relee_pins[0], LOW);
			digitalWrite_328(relee_pins[1], HIGH);
			break;
		case 3:
			digitalWrite_328(relee_pins[0], LOW);
			digitalWrite_328(relee_pins[1], LOW);
			break;
	}

}



void service(void)
{

	static enum { waiting_for_connection, set_vss_pin, set_charge_on, waiting_while_charging, removing_vss } position;
	static Timer t1;
	static char voltage[4];
	//static char loc_pin_no;
	static char current_vss_loc_pin_no;
	static int init = 0;

	// Initialize
	if(!init)
	{
		//if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "S - init\r\n");
		for(int i = 2; i < 13; i++)
		{
			pinMode_328(i, OUTPUT);
			digitalWrite_328(i, LOW);
		}
		t1 = newTimer_328();
		position = waiting_for_connection;
		init = 1;
	}

	switch(position)
	{
		case waiting_for_connection:
			if(receivedSignal(drone_connected, &voltage[0], &voltage[1], &voltage[2], &voltage[3]))
			{
				//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - waiting - Drone connected, setting relee\r\n");

				if(!voltage[0])
				{
					set_relee(0);
					current_vss_loc_pin_no = 0;
				}
				else if (!voltage[1])
				{
					set_relee(1);
					current_vss_loc_pin_no = 1;
				}
				else if (!voltage[2])
				{
					set_relee(2);
					current_vss_loc_pin_no = 2;
				}
				else if (!voltage[3])
				{
					set_relee(3);
					current_vss_loc_pin_no = 3;
				}

			position = set_vss_pin;
			startTimer_328(WAIT_TIME, t1);

			}
			break;

		case set_vss_pin:
			if(timeout_328(t1))
			{
				//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - set_vss_pin - connecting vss and turning on leds\r\n");
				set_vss_and_leds(current_vss_loc_pin_no);
				position = set_charge_on;
				startTimer_328(WAIT_TIME, t1);
			}
			break;

		case set_charge_on:
			if (timeout_328(t1))
			{
				//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - set charge on - connection chargers\r\n");
				set_charge(1);
				position = waiting_while_charging;
			}
			break;

		case waiting_while_charging:
			if(receivedSignal(done_charging))
			{
				//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - Charging done/interrupted - remove chargers\r\n");
				set_charge(0);
				startTimer_328(WAIT_TIME, t1);
				position = removing_vss;
			}

			break;

		case removing_vss:
			if(timeout_328(t1))
			{
				//if (DEBUG && SERVICE_DEBUG) usartPrint_328(DEBUG_PORT, "S - remove vss - removing vss, and turning off leds\r\n");
				set_vss_and_leds(4);

				position = waiting_for_connection;

			}
			break;

	}


}

void service_test_relee(void)
{
	static Timer t1;
	static char init = 0;
	static char i = 0;
	// Initialize
	if(!init)
	{
		for(int i = 2; i < 13; i++)
		{
			pinMode_328(i, OUTPUT);
			digitalWrite_328(i, LOW);
		}
		t1 = newTimer_328();
		init = 1;
		startTimer_328(100, t1);
	}

	if (timeout_328(t1))
	{
		startTimer_328(2000, t1);

		set_relee(i);
		//set_vss_and_leds(i);
		i++;
		if (i>3) i = 0;
	}

}

